@@ -9,14 +9,13 @@
 #include <dune/common/reservedvector.hh>
 #include <dune/common/classname.hh>
 #include <dune/common/fvector.hh>
+#include <dune/common/indices.hh>
 #include <dune/common/tuplevector.hh>
 #include <dune/common/test/testsuite.hh>
 
 #include <dune/istl/bvector.hh>
 #include <dune/istl/multitypeblockvector.hh>
 
-#include <dune/typetree/utility.hh>
-
 #include <dune/functions/common/type_traits.hh>
 #include <dune/functions/functionspacebases/hierarchicvectorwrapper.hh>
 
@@ -130,7 +129,7 @@ Dune::TestSuite checkHierarchicVector(std::string shortName="")
 {
   Dune::TestSuite test(shortName);
 
-  using namespace Dune::TypeTree::Indices;
+  using namespace Dune::Indices;
   using SizeInfo = HybridSizeInfoDummy<dim>;
   using SizePrefix = typename SizeInfo::SizePrefix;
 
