@@ -4,7 +4,7 @@
 
 #include <iostream>
 
-#include <dune/common/unused.hh>
+#include <dune/common/indices.hh>
 #include <dune/common/exceptions.hh>
 #include <dune/common/parallel/mpihelper.hh>
 #include <dune/common/test/testsuite.hh>
@@ -40,7 +40,7 @@ int main (int argc, char* argv[]) try
   Basis feBasis(gridView);
 
 
-  test.subTest(checkBasis(feBasis));
+  test.subTest(checkBasis(feBasis, EnableContinuityCheck()));
 
 
   typedef Basis::MultiIndex MultiIndex;
@@ -79,10 +79,10 @@ int main (int argc, char* argv[]) try
     }
 
     // get access to the finite element
-    using namespace Dune::TypeTree::Indices;
-    typedef Basis::LocalView::Tree Tree DUNE_UNUSED;
+    using namespace Dune::Indices;
+    using Tree [[maybe_unused]] = Basis::LocalView::Tree;
     auto& p_leaf = TypeTree::child(localView.tree(),_1);
-    auto& v_leaf DUNE_UNUSED = TypeTree::child(localView.tree(),_0,1);
+    [[maybe_unused]] auto& v_leaf = TypeTree::child(localView.tree(),_0,1);
     auto& localFiniteElement = p_leaf.finiteElement();
 
     // A quadrature rule
@@ -124,14 +124,14 @@ int main (int argc, char* argv[]) try
     typedef GridType::LeafGridView GridView;
     const GridView& gridView = grid.leafGridView();
     Functions::TaylorHoodBasis<GridView> basis(gridView);
-    test.subTest(checkBasis(basis));
+    test.subTest(checkBasis(basis, EnableContinuityCheck()));
   }
 
   // check RaviartThomasBasis created using basis builder mechanism
   {
     using namespace Functions::BasisFactory;
     auto basis = makeBasis(grid.leafGridView(), taylorHood());
-    test.subTest(checkBasis(basis));
+    test.subTest(checkBasis(basis, EnableContinuityCheck()));
   }
 
   return test.exit();
