@@ -82,10 +82,9 @@ void getLocalMatrix(
   {
     // { begin_quad_loop_end }
     // { quad_loop_preamble_begin }
-    // The transposed inverse Jacobian of the map from the
+    // The inverse Jacobian of the map from the
     // reference element to the element
-    const auto jacobianInverseTransposed
-            = geometry.jacobianInverseTransposed(quadPoint.position());
+    const auto jacobianInverse = geometry.jacobianInverse(quadPoint.position());
 
     // The multiplicative factor in the integral transformation formula
     const auto integrationElement
@@ -106,7 +105,7 @@ void getLocalMatrix(
     // Compute the shape function gradients on the grid element
     std::vector<FieldMatrix<double,1,dim> > jacobians(referenceJacobians.size());
     for (size_t i=0; i<jacobians.size(); i++)
-      jacobians[i] = referenceJacobians[i] * transpose(jacobianInverseTransposed);
+      jacobians[i] = referenceJacobians[i] * jacobianInverse;
     // { velocity_gradients_end }
 
     // Compute the actual matrix entries
@@ -233,9 +232,6 @@ template<class Matrix, class MultiIndex>
 decltype(auto) matrixEntry(Matrix& matrix, const MultiIndex& row, const MultiIndex& col)
 {
   return matrix[row[0]][col[0]][row[1]][col[1]];
-  using namespace Functions::BasisFactory;
-
-  static const std::size_t K = 1; // pressure order for Taylor-Hood
 }
 #endif
 
@@ -362,6 +358,10 @@ int main (int argc, char *argv[]) try
   using PressureVector = BlockVector<FieldVector<double,1>>;
   using VectorType = MultiTypeBlockVector<VelocityVector, PressureVector>;
 
+  using VelocityBitVector = BlockVector<FieldVector<char,dim>>;
+  using PressureBitVector = BlockVector<FieldVector<char,1>>;
+  using BitVectorType = MultiTypeBlockVector<VelocityBitVector, PressureBitVector>;
+
   using Matrix00 = BCRSMatrix<FieldMatrix<double,dim,dim>>;
   using Matrix01 = BCRSMatrix<FieldMatrix<double,dim,1>>;
   using Matrix10 = BCRSMatrix<FieldMatrix<double,1,dim>>;
@@ -400,10 +400,6 @@ int main (int argc, char *argv[]) try
   /////////////////////////////////////////////////////////
 
   // { initialize_boundary_dofs_vector_begin }
-  using VelocityBitVector = BlockVector<FieldVector<char,dim>>;
-  using PressureBitVector = BlockVector<FieldVector<char,1>>;
-  using BitVectorType
-          = MultiTypeBlockVector<VelocityBitVector, PressureBitVector>;
 
   BitVectorType isBoundary;
 
@@ -476,7 +472,7 @@ int main (int argc, char *argv[]) try
   // Construct the actual iterative solver
   RestartedGMResSolver<VectorType> solver(
           stiffnessOperator,  // operator to invert
-          preconditioner,     // preconditioner for interation
+          preconditioner,     // preconditioner for iteration
           1e-10,              // desired residual reduction factor
           500,                // number of iterations between restarts
           500,                // maximum number of iterations
